library(SKAT)
setwd("~/Project/Kernel_Machine/R_Package/MetaSKAT/test/Datasets")


data(SKAT.haplotypes)
attach(SKAT.haplotypes)
##################################
# Parameters

set.seed(100)
Total_Region_L<-200000	# total 200K bps region
Region_L<-5000			# test region size = 5k bps
N<-997					# Sample size=1000

####################################
# Markers
# Rare variants : MAF < 0.03
# Common variants : MAF >= 0.03

INFO<-SNPInfo
IDX_SNP<-which(INFO$FREQ1 < 0.5)


####################################
# Datasets 3 cohorts, 10 Sets

SNP_IDX<-list()
IDX_ALL<-NULL
for(i in 1:10){


	Region.Start<-runif(1)*(Total_Region_L - Region_L)
	Region.End<-Region.Start + Region_L

	Marker.Idx1<-which(INFO$CHROM_POS >= Region.Start)
	Marker.Idx2<-which(INFO$CHROM_POS <= Region.End)
	Marker.Idx3<-sort(intersect(Marker.Idx1,Marker.Idx2))
    
	IDX<-intersect(IDX_SNP, Marker.Idx3)
	SNP_IDX[[i]]<-IDX
	IDX_ALL<-c(IDX_ALL, IDX)
}

IDX_ALL<-sort(unique(IDX_ALL))

OUT_Cohort<-list()
OUT_Cohort1<-list()
for(j in 1:3){

	
	ID1<-sample(1:10000, N)
	ID2<-sample(1:10000, N)
		
	OUT_Cohort[[j]]<-list()
	for(i in 1:10){
		
		IDX<-SNP_IDX[[i]]
		Z<-Haplotype[ID1, IDX] + Haplotype[ID2, IDX]
		
		IDX1<-sample(1:length(IDX))
		MAF<-colMeans(Z[,IDX1])/2
		id<-which(MAF > 0)	
		IDX2<-IDX1[id]
		IDX3<-IDX[IDX2]
		
		OUT_Cohort[[j]][[i]]<-list(Z=Z, Z.r=Z[,IDX2], Info=INFO[IDX,], Info.r=INFO[IDX3,])

	}
	Z_all<-Haplotype[ID1, IDX_ALL] + Haplotype[ID2, IDX_ALL]
	OUT_Cohort1[[j]]<-list(Z_all=Z_all, y=rnorm(N))
	
}

save(OUT_Cohort, OUT_Cohort1, file="./out.Rdata")

#########################################
# Plink Files - Save


nMarker<-length(IDX_ALL)
for(i in 1:3){
	FAM<-cbind(1:N,rep(0,N),rep(0,N),rep(0,N),rep(1,N),OUT_Cohort1[[i]]$y)
	rs<-sprintf("SNP%09d",INFO[IDX_ALL,1])
	MAP<-data.frame(chr=rep(1,nMarker),id=rs,dist1=rep(0,nMarker),dist2=INFO[IDX_ALL,3])

	G1<-matrix(rep("A",N*nMarker),nrow=N)
	G2<-matrix(rep("A",N*nMarker),nrow=N)
	
	Z1<-OUT_Cohort1[[i]]$Z_all
	IDX1<-which(Z1==1)
	IDX2<-which(Z1==2)
	
	G1[IDX1]<-"T"
	G1[IDX2]<-"T"
	G2[IDX2]<-"T"
	
	Geno<-matrix(rep("A",N*nMarker*2),nrow=N)
	id1<-(1:nMarker) *2 -1
	id2<-(1:nMarker) *2 
	
	Geno[,id1]<-G1
	Geno[,id2]<-G2
	
	Geno.all<-cbind(FAM, Geno)
	
	File.Ped<-sprintf("./PedFiles/%02d.ped",i)
	File.Map<-sprintf("./PedFiles/%02d.map",i)
	File.SetID1<-sprintf("%02d_1.SetID",i)
	File.SetID2<-sprintf("%02d_2.SetID",i)
	File.SetID3<-sprintf("%02d_3.SetID",i)
	File.SetID4<-sprintf("%02d_4.SetID",i)
	
	write.table(Geno.all, File.Ped, col.names=FALSE, row.names=FALSE, quote=FALSE, sep=" ")
	write.table(MAP, File.Map, col.names=FALSE, row.names=FALSE, quote=FALSE, sep=" ")
	
	append=FALSE	
	append1=FALSE
	
	for(k in 1:10){
	
		
		rs<-sprintf("SNP%09d",(k*10000):(k*10000 + 10))
		temp1<-data.frame(SetID=rep(k*100, length(rs)),id=rs)
		write.table(temp1, File.SetID3, col.names=FALSE, row.names=FALSE, quote=FALSE, sep=" ", append=append1)
		append1=TRUE
		
		info1<-OUT_Cohort[[i]][[k]]$ Info
		info2<-OUT_Cohort[[i]][[k]]$ Info.r

 		rs<-sprintf("SNP%09d",info1[,1])
		temp1<-data.frame(SetID=rep(k*1000, length(rs)),id=rs)

 		rs<-sprintf("SNP%09d",info2[,1])
		temp2<-data.frame(SetID=rep(k*1000, length(rs)),id=rs)
		
		rs<-sprintf("SNP%09d",info2[,1])
		temp3<-data.frame(SetID=rep(k*1000, 1),id=rs[1])
		
		
		write.table(temp1, File.SetID1, col.names=FALSE, row.names=FALSE, quote=FALSE, sep=" ", append=append)
		write.table(temp2, File.SetID2, col.names=FALSE, row.names=FALSE, quote=FALSE, sep=" ", append=append)
		write.table(temp1, File.SetID3, col.names=FALSE, row.names=FALSE, quote=FALSE, sep=" ", append=append1)
		write.table(temp3, File.SetID4, col.names=FALSE, row.names=FALSE, quote=FALSE, sep=" ", append=append)
		
		append=TRUE
		
						
	}
	
}



